@@ -14,6 +14,9 @@ use function unlink;
 use BankAccount;
 use RecursiveDirectoryIterator;
 use RecursiveIteratorIterator;
+use SebastianBergmann\CodeCoverage\Data\ProcessedBranchCoverageData;
+use SebastianBergmann\CodeCoverage\Data\ProcessedFunctionCoverageData;
+use SebastianBergmann\CodeCoverage\Data\ProcessedPathCoverageData;
 use SebastianBergmann\CodeCoverage\Data\RawCodeCoverageData;
 use SebastianBergmann\CodeCoverage\Driver\Driver;
 use SebastianBergmann\CodeCoverage\Test\Target\Target;
@@ -1028,6 +1031,7 @@ abstract class TestCase extends \PHPUnit\Framework\TestCase
                     Target::forMethod(BankAccount::class, 'getBalance'),
                 ],
             ),
+            time: 0.1,
         );
 
         $coverage->start(
@@ -1042,6 +1046,7 @@ abstract class TestCase extends \PHPUnit\Framework\TestCase
                     Target::forMethod(BankAccount::class, 'withdrawMoney'),
                 ],
             ),
+            time: 0.2,
         );
 
         $coverage->start(
@@ -1056,6 +1061,7 @@ abstract class TestCase extends \PHPUnit\Framework\TestCase
                     Target::forMethod(BankAccount::class, 'depositMoney'),
                 ],
             ),
+            time: 0.3,
         );
 
         $coverage->start(
@@ -1072,6 +1078,7 @@ abstract class TestCase extends \PHPUnit\Framework\TestCase
                     Target::forMethod(BankAccount::class, 'withdrawMoney'),
                 ],
             ),
+            time: 0.4,
         );
 
         return $coverage;
@@ -1107,6 +1114,7 @@ abstract class TestCase extends \PHPUnit\Framework\TestCase
                     Target::forMethod(BankAccount::class, 'getBalance'),
                 ],
             ),
+            time: 0.5,
         );
 
         $coverage->start(
@@ -1121,6 +1129,7 @@ abstract class TestCase extends \PHPUnit\Framework\TestCase
                     Target::forMethod(BankAccount::class, 'withdrawMoney'),
                 ],
             ),
+            time: 0.6,
         );
 
         $coverage->start(
@@ -1135,6 +1144,7 @@ abstract class TestCase extends \PHPUnit\Framework\TestCase
                     Target::forMethod(BankAccount::class, 'depositMoney'),
                 ],
             ),
+            time: 0.7,
         );
 
         $coverage->start(
@@ -1151,6 +1161,7 @@ abstract class TestCase extends \PHPUnit\Framework\TestCase
                     Target::forMethod(BankAccount::class, 'withdrawMoney'),
                 ],
             ),
+            time: 0.8,
         );
 
         return $coverage;
@@ -1273,6 +1284,7 @@ abstract class TestCase extends \PHPUnit\Framework\TestCase
             TargetCollection::fromArray([
                 Target::forMethod(BankAccountTrait::class, 'withdrawMoney'),
             ]),
+            time: 0.9,
         );
 
         $coverage->start(
@@ -1285,6 +1297,7 @@ abstract class TestCase extends \PHPUnit\Framework\TestCase
             TargetCollection::fromArray([
                 Target::forMethod(BankAccountTrait::class, 'depositMoney'),
             ]),
+            time: 1.0,
         );
 
         $coverage->start(
@@ -1299,6 +1312,7 @@ abstract class TestCase extends \PHPUnit\Framework\TestCase
                 Target::forMethod(BankAccountTrait::class, 'depositMoney'),
                 Target::forMethod(BankAccountTrait::class, 'withdrawMoney'),
             ]),
+            time: 1.1,
         );
 
         return $coverage;
@@ -1332,6 +1346,7 @@ abstract class TestCase extends \PHPUnit\Framework\TestCase
                     Target::forMethod(BankAccount::class, 'getBalance'),
                 ],
             ),
+            time: 1.2,
         );
 
         $coverage->start(
@@ -1346,6 +1361,7 @@ abstract class TestCase extends \PHPUnit\Framework\TestCase
                     Target::forMethod(BankAccount::class, 'withdrawMoney'),
                 ],
             ),
+            time: 1.3,
         );
 
         return $coverage;
@@ -1377,6 +1393,7 @@ abstract class TestCase extends \PHPUnit\Framework\TestCase
                     Target::forMethod(BankAccount::class, 'depositMoney'),
                 ],
             ),
+            time: 1.4,
         );
 
         $coverage->start(
@@ -1393,6 +1410,7 @@ abstract class TestCase extends \PHPUnit\Framework\TestCase
                     Target::forMethod(BankAccount::class, 'withdrawMoney'),
                 ],
             ),
+            time: 1.5,
         );
 
         return $coverage;
@@ -1488,6 +1506,7 @@ abstract class TestCase extends \PHPUnit\Framework\TestCase
                     Target::forMethod(BankAccount::class, 'getBalance'),
                 ],
             ),
+            time: 1.6,
         );
 
         $coverage->start(
@@ -1502,6 +1521,7 @@ abstract class TestCase extends \PHPUnit\Framework\TestCase
                     Target::forMethod(BankAccount::class, 'withdrawMoney'),
                 ],
             ),
+            time: 1.7,
         );
 
         return $coverage;
@@ -1535,6 +1555,7 @@ abstract class TestCase extends \PHPUnit\Framework\TestCase
                     Target::forMethod(BankAccount::class, 'depositMoney'),
                 ],
             ),
+            time: 1.8,
         );
 
         $coverage->start(
@@ -1551,6 +1572,7 @@ abstract class TestCase extends \PHPUnit\Framework\TestCase
                     Target::forMethod(BankAccount::class, 'withdrawMoney'),
                 ],
             ),
+            time: 1.9,
         );
 
         return $coverage;
@@ -1560,201 +1582,199 @@ abstract class TestCase extends \PHPUnit\Framework\TestCase
     {
         return [
             TEST_FILES_PATH . 'BankAccount.php' => [
-                'BankAccount->depositMoney' => [
-                    'branches' => [
-                        0 => [
-                            'op_start'   => 0,
-                            'op_end'     => 14,
-                            'line_start' => 20,
-                            'line_end'   => 25,
-                            'hit'        => [
+                'BankAccount->depositMoney' => new ProcessedFunctionCoverageData(
+                    [
+                        0 => new ProcessedBranchCoverageData(
+                            0,
+                            14,
+                            20,
+                            25,
+                            [
                                 0 => 'BankAccountTest::testBalanceCannotBecomeNegative2',
                                 1 => 'BankAccountTest::testDepositWithdrawMoney',
                             ],
-                            'out' => [
-                            ],
-                            'out_hit' => [
-                            ],
-                        ],
+                            [],
+                            [],
+                        ),
                     ],
-                    'paths' => [
-                        0 => [
-                            'path' => [
+                    [
+                        0 => new ProcessedPathCoverageData(
+                            [
                                 0 => 0,
                             ],
-                            'hit' => [
+                            [
                                 0 => 'BankAccountTest::testBalanceCannotBecomeNegative2',
                                 1 => 'BankAccountTest::testDepositWithdrawMoney',
                             ],
-                        ],
+                        ),
                     ],
-                ],
-                'BankAccount->getBalance' => [
-                    'branches' => [
-                        0 => [
-                            'op_start'   => 0,
-                            'op_end'     => 5,
-                            'line_start' => 6,
-                            'line_end'   => 9,
-                            'hit'        => [
+                ),
+                'BankAccount->getBalance' => new ProcessedFunctionCoverageData(
+                    [
+                        0 => new ProcessedBranchCoverageData(
+                            0,
+                            5,
+                            6,
+                            9,
+                            [
                                 0 => 'BankAccountTest::testBalanceIsInitiallyZero',
                                 1 => 'BankAccountTest::testDepositWithdrawMoney',
                             ],
-                            'out' => [
+                            [
                             ],
-                            'out_hit' => [
+                            [
                             ],
-                        ],
+                        ),
                     ],
-                    'paths' => [
-                        0 => [
-                            'path' => [
+                    [
+                        0 => new ProcessedPathCoverageData(
+                            [
                                 0 => 0,
                             ],
-                            'hit' => [
+                            [
                                 0 => 'BankAccountTest::testBalanceIsInitiallyZero',
                                 1 => 'BankAccountTest::testDepositWithdrawMoney',
                             ],
-                        ],
+                        ),
                     ],
-                ],
-                'BankAccount->withdrawMoney' => [
-                    'branches' => [
-                        0 => [
-                            'op_start'   => 0,
-                            'op_end'     => 14,
-                            'line_start' => 27,
-                            'line_end'   => 32,
-                            'hit'        => [
+                ),
+                'BankAccount->withdrawMoney' => new ProcessedFunctionCoverageData(
+                    [
+                        0 => new ProcessedBranchCoverageData(
+                            0,
+                            14,
+                            27,
+                            32,
+                            [
                                 0 => 'BankAccountTest::testBalanceCannotBecomeNegative',
                                 1 => 'BankAccountTest::testDepositWithdrawMoney',
                             ],
-                            'out' => [
+                            [
                             ],
-                            'out_hit' => [
+                            [
                             ],
-                        ],
+                        ),
                     ],
-                    'paths' => [
-                        0 => [
-                            'path' => [
+                    [
+                        0 => new ProcessedPathCoverageData(
+                            [
                                 0 => 0,
                             ],
-                            'hit' => [
+                            [
                                 0 => 'BankAccountTest::testBalanceCannotBecomeNegative',
                                 1 => 'BankAccountTest::testDepositWithdrawMoney',
                             ],
-                        ],
+                        ),
                     ],
-                ],
-                '{main}' => [
-                    'branches' => [
-                        0 => [
-                            'op_start'   => 0,
-                            'op_end'     => 1,
-                            'line_start' => 34,
-                            'line_end'   => 34,
-                            'hit'        => [
+                ),
+                '{main}' => new ProcessedFunctionCoverageData(
+                    [
+                        0 => new ProcessedBranchCoverageData(
+                            0,
+                            1,
+                            34,
+                            34,
+                            [
                             ],
-                            'out' => [
+                            [
                                 0 => 2147483645,
                             ],
-                            'out_hit' => [
+                            [
                                 0 => 0,
                             ],
-                        ],
+                        ),
                     ],
-                    'paths' => [
-                        0 => [
-                            'path' => [
+                    [
+                        0 => new ProcessedPathCoverageData(
+                            [
                                 0 => 0,
                             ],
-                            'hit' => [
+                            [
                             ],
-                        ],
+                        ),
                     ],
-                ],
-                'BankAccount->setBalance' => [
-                    'branches' => [
-                        0 => [
-                            'op_start'   => 0,
-                            'op_end'     => 4,
-                            'line_start' => 11,
-                            'line_end'   => 13,
-                            'hit'        => [
+                ),
+                'BankAccount->setBalance' => new ProcessedFunctionCoverageData(
+                    [
+                        0 => new ProcessedBranchCoverageData(
+                            0,
+                            4,
+                            11,
+                            13,
+                            [
                             ],
-                            'out' => [
+                            [
                                 0 => 5,
                                 1 => 9,
                             ],
-                            'out_hit' => [
+                            [
                                 0 => 0,
                                 1 => 0,
                             ],
-                        ],
-                        5 => [
-                            'op_start'   => 5,
-                            'op_end'     => 8,
-                            'line_start' => 14,
-                            'line_end'   => 14,
-                            'hit'        => [
+                        ),
+                        5 => new ProcessedBranchCoverageData(
+                            5,
+                            8,
+                            14,
+                            14,
+                            [
                             ],
-                            'out' => [
+                            [
                                 0 => 13,
                             ],
-                            'out_hit' => [
+                            [
                                 0 => 0,
                             ],
-                        ],
-                        9 => [
-                            'op_start'   => 9,
-                            'op_end'     => 12,
-                            'line_start' => 16,
-                            'line_end'   => 16,
-                            'hit'        => [
+                        ),
+                        9 => new ProcessedBranchCoverageData(
+                            9,
+                            12,
+                            16,
+                            16,
+                            [
                             ],
-                            'out' => [
+                            [
                                 0 => 2147483645,
                             ],
-                            'out_hit' => [
+                            [
                                 0 => 0,
                             ],
-                        ],
-                        13 => [
-                            'op_start'   => 13,
-                            'op_end'     => 14,
-                            'line_start' => 18,
-                            'line_end'   => 18,
-                            'hit'        => [
+                        ),
+                        13 => new ProcessedBranchCoverageData(
+                            13,
+                            14,
+                            18,
+                            18,
+                            [
                             ],
-                            'out' => [
+                            [
                                 0 => 2147483645,
                             ],
-                            'out_hit' => [
+                            [
                                 0 => 0,
                             ],
-                        ],
+                        ),
                     ],
-                    'paths' => [
-                        0 => [
-                            'path' => [
+                    [
+                        0 => new ProcessedPathCoverageData(
+                            [
                                 0 => 0,
                                 1 => 5,
                                 2 => 13,
                             ],
-                            'hit' => [
+                            [
                             ],
-                        ],
-                        1 => [
-                            'path' => [
+                        ),
+                        1 => new ProcessedPathCoverageData(
+                            [
                                 0 => 0,
                                 1 => 9,
                             ],
-                            'hit' => [
+                            [
                             ],
-                        ],
+                        ),
                     ],
-                ],
+                ),
             ],
         ];
     }
@@ -1946,6 +1966,7 @@ abstract class TestCase extends \PHPUnit\Framework\TestCase
             TargetCollection::fromArray([
                 Target::forMethod(BankAccount::class, 'getBalance'),
             ]),
+            time: 2.0,
         );
 
         $coverage->start(
@@ -1958,6 +1979,7 @@ abstract class TestCase extends \PHPUnit\Framework\TestCase
             TargetCollection::fromArray([
                 Target::forMethod(BankAccount::class, 'withdrawMoney'),
             ]),
+            time: 2.1,
         );
 
         $coverage->start(
@@ -1970,6 +1992,7 @@ abstract class TestCase extends \PHPUnit\Framework\TestCase
             TargetCollection::fromArray([
                 Target::forMethod(BankAccount::class, 'depositMoney'),
             ]),
+            time: 2.2,
         );
 
         $coverage->start(
@@ -1984,6 +2007,7 @@ abstract class TestCase extends \PHPUnit\Framework\TestCase
                 Target::forMethod(BankAccount::class, 'depositMoney'),
                 Target::forMethod(BankAccount::class, 'withdrawMoney'),
             ]),
+            time: 2.3,
         );
 
         return $coverage;
@@ -2017,6 +2041,7 @@ abstract class TestCase extends \PHPUnit\Framework\TestCase
             TargetCollection::fromArray([
                 Target::forMethod(BankAccount::class, 'getBalance'),
             ]),
+            time: 2.4,
         );
 
         $coverage->start(
@@ -2029,6 +2054,7 @@ abstract class TestCase extends \PHPUnit\Framework\TestCase
             TargetCollection::fromArray([
                 Target::forMethod(BankAccount::class, 'withdrawMoney'),
             ]),
+            time: 2.5,
         );
 
         $coverage->start(
@@ -2041,6 +2067,7 @@ abstract class TestCase extends \PHPUnit\Framework\TestCase
             TargetCollection::fromArray([
                 Target::forMethod(BankAccount::class, 'depositMoney'),
             ]),
+            time: 2.6,
         );
 
         $coverage->start(
@@ -2055,6 +2082,7 @@ abstract class TestCase extends \PHPUnit\Framework\TestCase
                 Target::forMethod(BankAccount::class, 'depositMoney'),
                 Target::forMethod(BankAccount::class, 'withdrawMoney'),
             ]),
+            time: 2.7,
         );
 
         return $coverage;
