@@ -22,7 +22,7 @@ final class TextTest extends TestCase
         $text = new Text(Thresholds::default(), false, false);
 
         $this->assertStringMatchesFormatFile(
-            TEST_FILES_PATH . 'BankAccount-text-line.txt',
+            TEST_FILES_PATH . 'Report/Text/BankAccount-line.txt',
             str_replace(PHP_EOL, "\n", $text->process($this->getLineCoverageForBankAccount())),
         );
     }
@@ -32,7 +32,7 @@ final class TextTest extends TestCase
         $text = new Text(Thresholds::default(), false, false);
 
         $this->assertStringMatchesFormatFile(
-            TEST_FILES_PATH . 'BankAccount-text-path.txt',
+            TEST_FILES_PATH . 'Report/Text/BankAccount-path.txt',
             str_replace(PHP_EOL, "\n", $text->process($this->getPathCoverageForBankAccount())),
         );
     }
@@ -42,7 +42,7 @@ final class TextTest extends TestCase
         $text = new Text(Thresholds::default(), false, true);
 
         $this->assertStringMatchesFormatFile(
-            TEST_FILES_PATH . 'BankAccount-text-summary.txt',
+            TEST_FILES_PATH . 'Report/Text/BankAccount-summary.txt',
             str_replace(PHP_EOL, "\n", $text->process($this->getLineCoverageForBankAccount())),
         );
     }
@@ -52,7 +52,7 @@ final class TextTest extends TestCase
         $text = new Text(Thresholds::default(), true, false);
 
         $this->assertStringMatchesFormatFile(
-            TEST_FILES_PATH . 'NamespacedBankAccount-text.txt',
+            TEST_FILES_PATH . 'Report/Text/NamespacedBankAccount.txt',
             str_replace(PHP_EOL, "\n", $text->process($this->getLineCoverageForNamespacedBankAccount())),
         );
     }
@@ -62,7 +62,7 @@ final class TextTest extends TestCase
         $text = new Text(Thresholds::default(), true, false);
 
         $this->assertStringMatchesFormatFile(
-            TEST_FILES_PATH . 'NamespacedBankAccount-text-with-colors.txt',
+            TEST_FILES_PATH . 'Report/Text/NamespacedBankAccount-colors.txt',
             str_replace(PHP_EOL, "\n", $text->process($this->getLineCoverageForNamespacedBankAccount(), true)),
         );
     }
@@ -72,7 +72,7 @@ final class TextTest extends TestCase
         $text = new Text(Thresholds::default(), false, false);
 
         $this->assertStringMatchesFormatFile(
-            TEST_FILES_PATH . 'ignored-lines-text.txt',
+            TEST_FILES_PATH . 'Report/Text/ignored-lines.txt',
             str_replace(PHP_EOL, "\n", $text->process($this->getCoverageForFileWithIgnoredLines())),
         );
     }
@@ -82,7 +82,7 @@ final class TextTest extends TestCase
         $text = new Text(Thresholds::default(), false, false);
 
         $this->assertStringMatchesFormatFile(
-            TEST_FILES_PATH . 'class-with-anonymous-function-text.txt',
+            TEST_FILES_PATH . 'Report/Text/class-with-anonymous-function.txt',
             str_replace(PHP_EOL, "\n", $text->process($this->getCoverageForClassWithAnonymousFunction())),
         );
     }
@@ -92,7 +92,7 @@ final class TextTest extends TestCase
         $text = new Text(Thresholds::default(), false, false);
 
         $this->assertStringMatchesFormatFile(
-            TEST_FILES_PATH . 'BankAccountWithUncovered-text-line.txt',
+            TEST_FILES_PATH . 'Report/Text/BankAccountWithUncovered-line.txt',
             str_replace(PHP_EOL, "\n", $text->process($this->getCoverageForFilesWithUncoveredIncluded())),
         );
     }
@@ -102,7 +102,7 @@ final class TextTest extends TestCase
         $text = new Text(Thresholds::default(), false, false);
 
         $this->assertStringMatchesFormatFile(
-            TEST_FILES_PATH . 'BankAccountWithoutUncovered-text-line.txt',
+            TEST_FILES_PATH . 'Report/Text/BankAccountWithoutUncovered-line.txt',
             str_replace(PHP_EOL, "\n", $text->process($this->getCoverageForFilesWithUncoveredExcluded())),
         );
     }
